/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

Ext.Loader.setConfig({ enabled: true });
Ext.namespace('DSS.addon');

/** The gettext context for translation */
var gt_ctx = undefined;

/**
 * @class DSS.addon.Framework
 * @extends Ext.container.Viewport
 * Defines an abstract framework for dSS Addons.
 */
Ext.define('DSS.addon.Framework', {

    extend: 'Ext.container.Viewport',

    mixins: {
        jsonLogin: 'DSS.json.Login',
        jsonProperty: 'DSS.json.Property'
    },

    uses: [ 'Ext.Msg',
            'Ext.layout.container.Border',
            'Ext.layout.container.Fit'],

    config: {
        appName: '', // Don't forget to localize the name
        appIcon: 'images/dss/default_icon.png',
        appVersion: '',
        appId: '', // The root for relative property queries: /scripts/appId/
        disableGt: false,
        gtDomain: '', // The translation domain e.g. app_name
        gt: null
    },

    /**
     * Constructor to initialize the class and its dependencies.
     */
    constructor: function(config) {
        this.initConfig(config);

        this.mixins.jsonLogin.constructor.call(this.mixins.jsonLogin, config);
        // The hack is required only once since DSS.json.Login and DSS.json.Property
        // both inherit DSS.json.Base
        // Ext.applyIf(this, this.mixins.jsonLogin);
        this.mixins.jsonProperty.constructor.call(this.mixins.jsonProperty, config);
        // Hack to apply methods that the mixin inherits.
        // In this case DSS.json.Base methods
        Ext.applyIf(this, this.mixins.jsonProperty);

        this.callParent(arguments);
    },

    ///
    /// Abstract page functions
    ///

    /**
     * Gets the page content
     * @return {Object} Page content as Ext.container.Container object
     */
    getContent: function() { return undefined; },

    /**
     * Gets the page's help content
     * @return {Object} The page's help content as Ext.container.Container object
     */
    getHelp: function() { return undefined; },

    /**
     * Checks that the class has been implemented
     * @return {Array[String]} An array of error messages
     */
    checkImplementation: function() {
        var errors = [];
        if (this.config.appName == '')
            errors.push(_("App name is not set (field `appName')"));
        if (this.config.appVersion == '')
            errors.push(_("App version is not set (field `config.appVersion')"));
        if (this.config.appId == 0)
            errors.push(_("App ID is not set (field `config.appId')"));
        return errors;
    },

    ///
    /// Initialization
    ///

    /**
     * Initialize the general (static) layout of the page
     * before the rendering is done.
     * Use initPage() to kick off the dynamic parts that require json
     */
    initComponent: function() {
        var errors = this.checkImplementation();
        if (errors.length > 0) {
            Ext.Msg.alert (_('Addon implementation errors'), errors.join('<br>'));
        }

        document.title = document.title + " - " + this.config.name;

        // Check if gettext is disabled
        if (!this.disableGt) {
            var params = {
                            domain: this.gtDomain,
                            locale_data: { /* $(this.gtDomain): locale_data */ }
                         };
            if (typeof(locale_data) != 'undefined')
                params.locale_data[this.gtDomain] = locale_data;

            this.config.gt = new Gettext(params);
            gt_ctx = this.config.gt;
        }

        Ext.apply(this, {
            id: 'app-viewport',
            layout: {
                type: 'border',
                padding: '0 5 5 5' // pad the layout from the window edges
            },
            style: {
                backgroundColor: 'E9FFEE'
            },
            items: [{
                region: 'center',
                layout: 'border',
                border: false,
                items: [{
                    xtype: 'box',
                    region: 'north',
                    height: 80,
                    style: {
                        backgroundColor: 'E9FFEE',
                        lineHeight: '80px',
                        verticalAlign: 'middle'
                    },
                    html: '<table><tr><td><img id="logo" src="'
                            + this.appIcon
                            + '"></td><td width="100%"><h1>'
                            + this.appName + '</h1></td>'
                            + '<td><a href="http://digitalstrom.org/" alt="'
                            + _("visit digitalSTROM.org")
                            + '" target="_blank"><img id="logo" src="images/dss/ds_logo.png"'
                            + 'alt="dS-Logo"></a></td>'
                            + '</tr></table>'
                }, {
                    id: 'app-content',
                    region: 'center',
                    border: 0,
                    margins: '0 20 0 20',
                    layout: 'fit',
                    items: this.getContent(),
                }, {
                    id: 'app-help-content',
                    region: 'south',
                    // xtype: 'panel', // implied by default
                    title: _('Help'),
                    collapsible: true,
                    collapsed: true,
                    split: true, // enable resizing
                    minHeight: 100,
                    maxHeight: 400,
                    margins: '0 20 20 20',
                    padding: '5 5 0 5',
                    closeAction: 'hide',
                    autoScroll: true,
                    layout: 'fit',
                    items: this.getHelp(),
                }]
            }]
        });
        this.callParent(arguments);
    },

    /**
     * Initialize the dynamic parts of the page
     * after the initial rendering is done
     */
    initPage: function() {
        // Attach the default error handlers
        this.on({
            ajaxFailure: this.jsonAjaxFailure,
            loginFailed: this.jsonLoginFailed,
            queryError: this.jsonQueryError,
            responseError: this.jsonResponseError
        });

        // Remove the 'loading ...' message from the html
        var start = Ext.get('start');
        if (start)
            start.remove();

        // Authenticate the json connection
        this.login();
    },

    ///
    /// DSS helper functions
    ///

    jsonLoginFailed: function (err) {
        Ext.Msg.alert(_('Error'), _('Could not log in:') + '\n' + err);
    },

    jsonAjaxFailure: function (err) {
        Ext.Msg.alert(_('Error'), _('Could not send login command to dSS:') + '\n' + err);
    },

    jsonQueryError: function (err) {
        Ext.Msg.alert(_('Query error'), _('The query did not succeed:') + '\n' + err);
    },

    jsonResponseError: function (err) {
        Ext.Msg.alert(_('Remote response error'), _('The response could not be parsed:') + '\n' + err);
    }
});

/**
 * Gettext translation method
 * @param {String} msgid The string to translate
 * @param {String} arg1 Typically the plural form
 * @param {int} arg1 Typically the count
 * @return {String} The translated string if available, otherwise return msgid
 */
function _(msgid, arg1, arg2)
{
    if (gt_ctx) {
        if (arguments.length === 1)
            return gt_ctx.gettext(msgid);

        else if (arguments.length === 2)
            return gt_ctx.ngettext(msgid, arg1);

        else if (arguments.length === 3)
            return gt_ctx.ngettext(msgid, arg1, arg2);
    }
    return msgid;
}

